@@ -170,10 +170,45 @@ def _get_verse_num(verse_str: str) -> int:
 
 
 _ENCODINGS = {
+    37: "cp037",
+    437: "cp437",
+    500: "cp500",
+    720: "cp720",
+    737: "cp737",
+    775: "cp775",
+    850: "cp850",
+    852: "cp852",
+    855: "cp855",
+    857: "cp857",
+    858: "cp858",
+    860: "cp860",
+    861: "cp861",
+    862: "cp862",
+    863: "cp863",
+    864: "cp864",
+    865: "cp865",
+    866: "cp866",
+    869: "cp869",
+    874: "cp874",
+    875: "cp875",
+    932: "cp932",
     936: "gb2313",
+    949: "cp949",
+    950: "cp950",
+    1026: "cp1026",
+    1140: "cp1140",
     1200: "utf_16",
     1201: "utf_16_be",
+    1250: "cp1250",
+    1251: "cp1251",
     1252: "cp1252",
+    1253: "cp1253",
+    1254: "cp1254",
+    1255: "cp1255",
+    1256: "cp1256",
+    1257: "cp1257",
+    1258: "cp1258",
+    1361: "johab",
     12000: "utf_32",
     12001: "utf_32_be",
     20127: "ascii",
