@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from abc import abstractmethod
 from math import exp
 from pathlib import Path
@@ -8,10 +10,9 @@ import thot.alignment as ta
 from ...corpora.parallel_text_corpus import ParallelTextCorpus
 from ...utils.typeshed import StrPath
 from ..ibm1_word_alignment_model import Ibm1WordAlignmentModel
-from ..trainer import Trainer
 from ..word_alignment_matrix import WordAlignmentMatrix
 from ..word_vocabulary import WordVocabulary
-from .thot_utils import batch
+from .thot_utils import batch, escape_token, escape_tokens, unescape_token
 from .thot_word_alignment_model_trainer import ThotWordAlignmentModelTrainer
 from .thot_word_alignment_model_type import ThotWordAlignmentModelType
 from .thot_word_alignment_parameters import ThotWordAlignmentParameters
@@ -55,6 +56,8 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         ...
 
     def load(self, prefix_filename: StrPath) -> None:
+        if self._owned:
+            raise RuntimeError("The word alignment model is owned by an SMT model.")
         prefix_filename = Path(prefix_filename)
         if not (prefix_filename.parent / (prefix_filename.name + ".src")).is_file():
             raise FileNotFoundError("The word alignment model configuration could not be found.")
@@ -63,6 +66,8 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         self._model.load(str(prefix_filename))
 
     def create_new(self, prefix_filename: StrPath) -> None:
+        if self._owned:
+            raise RuntimeError("The word alignment model is owned by an SMT model.")
         self._prefix_filename = Path(prefix_filename)
         self._model.clear()
 
@@ -70,11 +75,15 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         if self._prefix_filename is not None:
             self._model.print(str(self._prefix_filename))
 
-    def create_trainer(self, corpus: ParallelTextCorpus) -> Trainer:
+    def create_trainer(self, corpus: ParallelTextCorpus) -> ThotWordAlignmentModelTrainer:
+        if self._owned:
+            raise RuntimeError("The word alignment model cannot be trained independently of its SMT model.")
         return _Trainer(self, corpus, self._prefix_filename)
 
     def align(self, source_segment: Sequence[str], target_segment: Sequence[str]) -> WordAlignmentMatrix:
-        _, matrix = self._model.get_best_alignment(source_segment, target_segment)
+        _, matrix = self._model.get_best_alignment(
+            list(escape_tokens(source_segment)), list(escape_tokens(target_segment))
+        )
         return WordAlignmentMatrix(matrix.to_numpy())
 
     def align_batch(self, segments: Sequence[Sequence[Sequence[str]]]) -> Sequence[WordAlignmentMatrix]:
@@ -101,11 +110,11 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         if source_word is None:
             source_word = 0
         elif isinstance(source_word, str):
-            source_word = self._model.get_src_word_index(source_word)
+            source_word = self._model.get_src_word_index(escape_token(source_word))
         if target_word is None or target_word == 0:
             return -99999
         elif isinstance(target_word, str):
-            target_word = self._model.get_trg_word_index(target_word)
+            target_word = self._model.get_trg_word_index(escape_token(target_word))
         return self._model.translation_log_prob(source_word, target_word)
 
     def get_sentence_length_probability(self, source_length: int, target_length: int) -> float:
@@ -120,9 +129,16 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         if source_word is None:
             source_word = 0
         elif isinstance(source_word, str):
-            source_word = self._model.get_src_word_index(source_word)
+            source_word = self._model.get_src_word_index(escape_token(source_word))
         return self._model.get_translations(source_word, threshold)
 
+    def close(self) -> None:
+        if not self._owned:
+            self._model.clear()
+
+    def __enter__(self) -> ThotWordAlignmentModel:
+        return self
+
     def _create_model(self) -> ta.AlignmentModel:
         if self.type is ThotWordAlignmentModelType.IBM1:
             return ta.Ibm1AlignmentModel()
@@ -139,8 +155,9 @@ class ThotWordAlignmentModel(Ibm1WordAlignmentModel):
         else:
             raise ValueError("The model type is invalid.")
 
-    def _set_model(self, model: ta.AlignmentModel) -> None:
+    def _set_model(self, model: ta.AlignmentModel, owned: bool = False) -> None:
         self._model = model
+        self._owned = owned
         self._source_words = _ThotWordVocabulary(self._model, is_src=True)
         self._target_words = _ThotWordVocabulary(self._model, is_src=False)
 
@@ -153,12 +170,15 @@ class _ThotWordVocabulary(WordVocabulary):
     def index(self, word: Optional[str]) -> int:
         if word is None:
             return 0
+        word = escape_token(word)
         return self._model.get_src_word_index(word) if self._is_src else self._model.get_trg_word_index(word)
 
     def __getitem__(self, word_index: int) -> str:
         if word_index >= len(self):
             raise IndexError
-        return self._model.get_src_word(word_index) if self._is_src else self._model.get_trg_word(word_index)
+        return unescape_token(
+            self._model.get_src_word(word_index) if self._is_src else self._model.get_trg_word(word_index)
+        )
 
     def __len__(self) -> int:
         return self._model.src_vocab_size if self._is_src else self._model.trg_vocab_size
@@ -183,3 +203,6 @@ class _Trainer(ThotWordAlignmentModelTrainer):
     def save(self) -> None:
         super().save()
         self._machine_model._set_model(self._model)
+
+    def close(self) -> None:
+        self._models.clear()
