@@ -1,5 +1,8 @@
-from abc import ABC, abstractmethod
-from typing import Callable, Dict, Optional
+from __future__ import annotations
+
+from abc import abstractmethod
+from types import TracebackType
+from typing import Callable, ContextManager, Dict, Optional, Type
 
 from ..utils.progress_status import ProgressStatus
 
@@ -14,7 +17,7 @@ class TrainStats:
         return self._metrics
 
 
-class Trainer(ABC):
+class Trainer(ContextManager["Trainer"]):
     @abstractmethod
     def train(
         self,
@@ -31,3 +34,17 @@ class Trainer(ABC):
     @abstractmethod
     def stats(self) -> TrainStats:
         ...
+
+    def close(self) -> None:
+        ...
+
+    def __enter__(self) -> Trainer:
+        return self
+
+    def __exit__(
+        self,
+        __exc_type: Optional[Type[BaseException]],
+        __exc_value: Optional[BaseException],
+        __traceback: Optional[TracebackType],
+    ) -> None:
+        self.close()
